import java.util.Vector;
import java.util.LinkedList;
import H.*;

class Histogram extends CoordinateSystem
{
    protected int m_numbins = 10;  // 10 bins by default 
    
    protected double m_minxdatapoint = Double.MAX_VALUE;
    protected double m_maxxdatapoint = -Double.MAX_VALUE;
    
    /*
     * this means that the high and low x will be the max and min
     * of the data set
     */
    protected boolean m_setRangeToData = true;

    /*
     * this is a vector of points with initial
     * size of 10000 and incrementing by 10000
     * as needed
     *
     * initially there are no points in the histogram
     */
    protected Vector m_points = new Vector(10000,10000);    
    protected long[] m_binCounts = new long[10];
    
    // overridden from class Drawable
    // so that we can clear the screenObjects list first
    protected void register()
    {
        H.pl("Registering the histogram object");
        Drawable.screenObjects = new LinkedList();
        Drawable.screenObjects.add(this);
    }
    
    public Histogram()
    {
        // let y go from 0 to 1 by default
        super(-1,1,0,1);
        // set the number of bins to be the width in pixels
        // one bin per pixel
        setNumBins(width);
    }
    
    public void addPoint(double x)
    {
        m_points.add(new Double(x));
        
        // we update the max and min of the data points
        // regardless of the scaling, just to have the
        // information around
        if (m_minxdatapoint > x)
            m_minxdatapoint = x;
            
        if (m_maxxdatapoint < x)
            m_maxxdatapoint = x;
        
        // if we're setting the range to the data
        // then we want to make sure the above values
        // are x_min and x_max...
        if (m_setRangeToData)
        {
            updateMovingRange();
        }
        
        // limits didn't move or we're not setting to data -- update the one bin
        addToBins(x);
    }
    
    public void setHistogramRange(double xmin, double xmax)
    {
        if (xmax > xmin)
        {
            m_setRangeToData = false;
            setCS(xmin,xmax,m_ymin,m_ymax);
            recomputeBins();   
        }
        else
        {
            H.pl("You can't set a histogram range with xmax <= xmin.");
            H.pl("An attempt to use xmin = " + xmin + " and xmax = " + xmax);
            H.pl("  has been ignored.");
        }
    }
    
    protected void updateMovingRange()
    {       
        // if the limits moved, update all bins
        if (m_minxdatapoint != m_xmin || m_maxxdatapoint != m_xmax)            
        {
            m_xmin = m_minxdatapoint;
            m_xmax = m_maxxdatapoint;
            recomputeBins();
        }
        // otherwise, do nothing
    }
    
    public int getNumBins() 
    { 
        return m_numbins; 
    }
    
    public void setNumBins(int m) 
    { 
        if (m>0 && m_numbins != m) 
        {
            m_numbins = m;
            m_binCounts = new long[m_numbins];
        }
        else
        {
            H.pl("Not changing the number of bins...");
        }
        recomputeBins();
    }
    
    public void setRangeToData(boolean b)
    {
        if (m_setRangeToData == b) return;
        
        // otherwise, set m_setRangeToData appropriately
        m_setRangeToData = b;
        
        // no need to recompute bins if nothing will change
        if (m_setRangeToData == false) // absolute range
            return;
            
        recomputeBins();                
    }
    
    public void handleResizeEvent()
    {
        super.handleResizeEvent();
        setNumBins(width);
    }
    
    protected void addToBins(double x)
    {
        //H.p("in addToBins");
        int bin = (int)(((x-m_xmin)/m_xdelta)*m_numbins);
        if (bin >= 0 && bin < m_numbins)
        {
            //H.pl("adding to bin " + bin);
            ++m_binCounts[bin];
        }
    }
    
    protected void recomputeBins()
    {            
        // get each point and increment the appropriate bincount
        // which is the appropriate bin? we need to know the
        // range in order to figure that out -- this is held in
        // m_xmin and m_xmax (and m_xdelta)
        double x = 0.0;   
        
        if (m_setRangeToData)
        {
            updateMovingRange();
        }
                 
        for (int i=0 ; i<m_points.size() ; ++i)
        {
            x = ((Double)(m_points.get(i))).doubleValue();
            addToBins(x);
        }
    }
    
    protected void scaleForHeight()
    {
        // go through the bins and set the axes height
        // to be about 10% more than the biggest bin
        int maxbinindex;
        long maxbinquantity = -1;
        
        for (int i=0 ; i<m_numbins ; ++i)
        {
            if (m_binCounts[i] > maxbinquantity)
            {
                maxbinquantity = m_binCounts[i];
                maxbinindex = i;
            }
        }
        
        setCS(m_xmin,m_xmax,0,(1.1*maxbinquantity/m_points.size()));
    }
    
    public void draw() 
    {
        // the coordinate system should be set up so that
        // we can step through the number of bins and fill
        // a rectangle from the top left endpoint down to 0
       
        scaleForHeight();
        H.out.setPrecision(3);
        H.p("Histogram bounds are   (");
        H.p(m_xmin);
        H.p(" < x < ");
        H.p(m_xmax);
        H.pl(")"); //  maxbinsize = " + maxbinsize);
        
        double ulx,uly;
        double barwidth = (1.0/m_numbins)*m_xdelta;
        
        for (int i=0 ; i<m_numbins ; ++i)
        {
            ulx = m_xmin + (i/(double)m_numbins)*m_xdelta;
            uly = m_binCounts[i]/((double)(m_points.size())); // percentage
            fillRect(ulx,uly,barwidth,uly); // fill down to 0
        }
        
    }
    
    
    
    
}

